#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2024 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import csv
import hashlib
import importlib.metadata
import logging
import os
from datetime import datetime
from pathlib import Path

import click

from .ena_queries import EnaQuery

logging.basicConfig(level=logging.INFO)

__version__ = importlib.metadata.version("assembly_uploader")


def parse_info(data_file):
    csvfile = open(data_file, newline="")
    csvdict = csv.DictReader(csvfile)
    return csvdict


def get_md5(path_to_file):
    md5_hash = hashlib.md5()
    with open(path_to_file, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            md5_hash.update(chunk)
    return md5_hash.hexdigest()


def create_assembly_alias(assembly_md5, runs, test=False):
    """
    Creates an alias for an assembly based on the MD5 of the FASTA file, runs,
    and an optional test flag.

    The alias consists of:
    1. The first run accession; appends '_others' if multiple runs exist (co-assembly).
    2. The first 12 characters of the MD5 hash of the assembly file for uniqueness.
    3. If `test` is True, a timestamp-based MD5 hash (first 8 characters) is added
        to make test submissions of the same day unique.
    4. Raises a ValueError if the alias exceeds 50 characters to comply with ENA standards.

    :param assembly_md5: The MD5 hash of the assembly FASTA file.
    :param runs: A list of run accessions.
    :param test: Flag indicating if the alias is for testing. Default is False.
    :return: The constructed assembly alias.
    :raises ValueError: If the alias exceeds 50 characters.
    """
    assembly_alias = (
        f"{runs[0]}{'_others' if len(runs) > 1 else ''}_{assembly_md5[:12]}"
    )

    if test:
        hash_part = hashlib.md5(datetime.now().isoformat().encode()).hexdigest()[:8]
        assembly_alias += f"_{hash_part}"

    if len(assembly_alias) > 50:
        raise ValueError(
            f"Assembly alias {assembly_alias} is too long. ENA allows maximum 50 characters."
        )

    return assembly_alias


class AssemblyManifestGenerator:
    def __init__(
        self,
        study: str,  # only used to name the upload directory
        assembly_study: str,
        assemblies_csv: Path,
        output_dir: Path = None,
        force: bool = False,
        private: bool = False,
        tpa: bool = False,
        test: bool = False,
    ):
        """
        Create an assembly manifest file for uploading assemblies detailed in assemblies_csv into the assembly_study.
        :param study: study accession of the raw reads study
        :param assembly_study: study accession of the assembly study (e.g. created by Study XMLs)
        :param assemblies_csv: path to assemblies CSV file, listing runs, coverage, assembler, version, filepath of each assembly
                            Optionally, a 'Sample' column can be included to specify sample accession for co-assemblies
        :param output_dir: path to output directory, otherwise CWD
        :param force: overwrite existing manifests
        :param private: is this a private study?
        :param tpa: is this a third-party assembly?

        """
        self.study = study
        self.metadata = parse_info(assemblies_csv)
        self.new_project = assembly_study

        self.upload_dir = (output_dir or Path(".")) / Path(f"{self.study}_upload")
        self.upload_dir.mkdir(exist_ok=True, parents=True)

        self.force = force
        self.private = private
        self.tpa = tpa
        self.test = test

    def generate_manifest(
        self,
        runs: list,
        sample: str,
        sequencer: str,
        coverage: str,
        assembler: str,
        assembler_version: str,
        assembly_path: Path,
    ) -> Path | None:
        """
        Generate a manifest file for submission to ENA.

        This method writes a manifest file for an assembly built from one or more sequencing runs.

        :param runs: Comma-separated list of ENA runs' accessions used in the assembly.
        :param sample: Sample accession. Can only be one sample accession, even for co-assemblies.
        :param sequencer: Instrument model used for sequencing.
        :param coverage: Reported coverage of the assembly.
        :param assembler: Name of the assembler used.
        :param assembler_version: Version of the assembler.
        :param assembly_path: Path to the assembly FASTA file (gzipped).

        """
        runs_str = ",".join(runs)

        logging.info(f"Writing manifest for {runs_str}")
        #   sanity check assembly file provided
        if not assembly_path.exists():
            logging.error(
                f"Assembly path {assembly_path} does not exist. Skipping manifest for run {runs_str}"
            )
            return None
        valid_extensions = (".fa.gz", ".fna.gz", ".fasta.gz")
        if not str(assembly_path).endswith(valid_extensions):
            logging.error(
                f"Assembly file {assembly_path} is either not fasta format or not compressed for run "
                f"{runs_str}."
            )
            return None
        assembly_md5 = get_md5(assembly_path)
        assembly_alias = create_assembly_alias(assembly_md5, runs, self.test)
        assembler = f"{assembler} v{assembler_version}"
        manifest_path = Path(self.upload_dir) / f"{assembly_md5[:12]}.manifest"
        #   skip existing manifests
        if os.path.exists(manifest_path) and not self.force:
            logging.warning(
                f"Manifest for {runs_str} already exists at {manifest_path}. Skipping"
            )
            return manifest_path
        values = (
            ("STUDY", self.new_project),
            ("SAMPLE", sample),
            ("RUN_REF", runs_str),
            ("ASSEMBLYNAME", assembly_alias),
            ("ASSEMBLY_TYPE", "primary metagenome"),
            ("COVERAGE", coverage),
            ("PROGRAM", assembler),
            ("PLATFORM", sequencer),
            ("FASTA", assembly_path),
            ("TPA", str(self.tpa).lower()),
        )
        logging.info("Writing manifest file (.manifest) for " + runs_str)
        with open(manifest_path, "w") as outfile:
            for k, v in values:
                manifest = f"{k}\t{v}\n"
                outfile.write(manifest)
        return manifest_path

    def write_manifests(self):
        for row in self.metadata:
            # collect sample accessions and instrument models from runs
            sample_accessions = set()
            instruments = set()
            for run in row["Runs"].split(","):
                # TODO in theory private/non-private state can be different for runs in co-assembly
                ena_query = EnaQuery(run, self.private)
                ena_metadata = ena_query.build_query()
                sample_accessions.add(ena_metadata["sample_accession"])
                instruments.add(ena_metadata["instrument_model"])

            # only one sample accession can be used for the assembly
            if len(sample_accessions) == 1:
                sample_accession = sample_accessions.pop()
            elif row.get("Sample"):
                # Use the explicitly provided sample accession
                sample_accession = row["Sample"]
            else:
                logging.error(
                    f"Multiple samples found for runs {row['Runs']}: {sample_accessions}. "
                    f"Please specify a sample accession in the 'Sample' column of your CSV to resolve this. Skipping."
                )
                continue

            self.generate_manifest(
                row["Runs"].split(","),
                sample_accession,
                ",".join(instruments),
                row["Coverage"],
                row["Assembler"],
                row["Version"],
                Path(row["Filepath"]),
            )

    # alias for convenience
    write = write_manifests


@click.command(help="Generate manifests for assembly uploads")
@click.version_option(__version__, message="assembly_uploader %(version)s")
@click.option(
    "--study",
    required=True,
    help="Raw reads study ID (used as a label for the upload directory)",
)
@click.option(
    "--data",
    type=click.Path(exists=True, dir_okay=False),
    required=False,
    help="Metadata CSV - runs, coverage, assembler, version, filepath, and optionally sample",
)
@click.option(
    "--assembly_study",
    required=False,
    help="Pre-existing study ID to submit to if available. Must exist in the webin account.",
)
@click.option(
    "--force", is_flag=True, default=False, help="Overwrite all existing manifests"
)
@click.option(
    "--output-dir",
    type=click.Path(file_okay=False, dir_okay=True, writable=True),
    required=False,
    help="Path to output directory",
)
@click.option("--private", is_flag=True, default=False, help="Use flag if private")
@click.option(
    "--tpa",
    is_flag=True,
    default=False,
    help="Use this flag if the study is a third-party assembly. Default: False",
)
@click.option(
    "--test",
    is_flag=True,
    default=False,
    help="Use flag when submitting to the ENA TEST server (adds a timestamp to the assembly alias)",
)
def main(study, assembly_study, data, force, private, tpa, output_dir, test):

    gen_manifest = AssemblyManifestGenerator(
        study=study,
        assembly_study=assembly_study,
        assemblies_csv=data,
        force=force,
        private=private,
        tpa=tpa,
        output_dir=output_dir,
        test=test,
    )
    gen_manifest.write_manifests()
    logging.info("Completed")


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2024 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import hashlib
import importlib.metadata
import xml.dom.minidom as minidom
import xml.etree.ElementTree as ET
from datetime import datetime
from pathlib import Path

import click

from .ena_queries import EnaQuery

METAGENOME = "metagenome"
METATRANSCRIPTOME = "metatranscriptome"


__version__ = importlib.metadata.version("assembly_uploader")


class StudyXMLGenerator:
    def __init__(
        self,
        study: str,
        center_name: str,
        library: str,
        hold_date: datetime = None,
        tpa: bool = False,
        output_dir: Path = None,
        publication: int = None,
        private: bool = False,
        test: bool = False,
    ):
        f"""
        Build submission files for an assembly study.

        :param study: raw reads study ID/accession
        :param center_name: submission centre name, e.g. EMG
        :param library: {METAGENOME} or {METATRANSCRIPTOME}
        :param hold_date: hold date for the data to remain private, if it should be different from the provided study"
        :param tpa: is this a third-party assembly?
        :param output_dir: path to output directory (default is CWD)
        :param publication: pubmed ID for connected publication if available
        :param private: is this a private study?
        :param test: submition will be done to the ENA TEST server
        :return: StudyXMLGenerator object
        """
        self.study = study

        self.upload_dir = (output_dir or Path(".")) / Path(f"{self.study}_upload")
        self.upload_dir = self.upload_dir.absolute()
        self.upload_dir.mkdir(parents=True, exist_ok=True)

        self.study_xml_path = self.upload_dir / Path(f"{self.study}_reg.xml")
        self.submission_xml_path = self.upload_dir / Path(
            f"{self.study}_submission.xml"
        )

        self.center = center_name
        self.hold_date = hold_date

        assert library in [METAGENOME, METATRANSCRIPTOME]

        self.library = library
        self.tpa = tpa
        self.publication = publication
        self.private = private
        self.test = test

        ena_query = EnaQuery(self.study, self.private)
        self.study_obj = ena_query.build_query()

        self._title = None
        self._abstract = None

    def write_study_xml(self):
        subtitle = self.library.title()
        if self.tpa:
            sub_abstract = "Third Party Annotation (TPA) "
        else:
            sub_abstract = ""

        title = (
            f"{subtitle} assembly of {self.study_obj['study_accession']} data "
            f"set ({self.study_obj['study_title']})"
        )
        self._title = title
        abstract = (
            f"The {sub_abstract}assembly was derived from the primary data "
            f"set {self.study_obj['study_accession']}"
        )
        self._abstract = abstract

        project_alias = self.study_obj["study_accession"] + "_assembly"
        if self.test:
            # add timestamp to be able to test multiple submissions during the same day
            hash_part = hashlib.md5(datetime.now().isoformat().encode()).hexdigest()[:8]
            project_alias += f"_{hash_part}"
        with open(self.study_xml_path, "wb") as study_file:
            project_set = ET.Element("PROJECT_SET")
            project = ET.SubElement(project_set, "PROJECT")
            project.set("alias", project_alias)
            project.set("center_name", self.center)

            ET.SubElement(project, "TITLE").text = title
            ET.SubElement(project, "DESCRIPTION").text = abstract

            # submission
            submission_project = ET.SubElement(project, "SUBMISSION_PROJECT")
            ET.SubElement(submission_project, "SEQUENCING_PROJECT")

            # publication links
            if self.publication:
                project_links = ET.SubElement(project, "PROJECT_LINKS")
                project_link = ET.SubElement(project_links, "PROJECT_LINK")
                xref_link = ET.SubElement(project_link, "XREF_LINK")
                ET.SubElement(xref_link, "DB").text = "PUBMED"
                ET.SubElement(xref_link, "ID").text = str(self.publication)

            # project attributes: TPA and assembly type
            project_attributes = ET.SubElement(project, "PROJECT_ATTRIBUTES")
            if self.tpa:
                project_attribute_tpa = ET.SubElement(
                    project_attributes, "PROJECT_ATTRIBUTE"
                )
                ET.SubElement(project_attribute_tpa, "TAG").text = "study keyword"
                ET.SubElement(project_attribute_tpa, "VALUE").text = "TPA:assembly"

            project_attribute_type = ET.SubElement(
                project_attributes, "PROJECT_ATTRIBUTE"
            )
            ET.SubElement(project_attribute_type, "TAG").text = "new_study_type"
            ET.SubElement(project_attribute_type, "VALUE").text = (
                f"{self.library} assembly"
            )

            dom = minidom.parseString(ET.tostring(project_set, encoding="utf-8"))
            study_file.write(dom.toprettyxml().encode("utf-8"))

    def write_submission_xml(self):
        with open(self.submission_xml_path, "wb") as submission_file:
            submission = ET.Element("SUBMISSION")
            submission.set("center_name", self.center)

            # template
            actions = ET.SubElement(submission, "ACTIONS")
            action_sub = ET.SubElement(actions, "ACTION")
            ET.SubElement(action_sub, "ADD")

            # attributes: function and hold date
            public = self.study_obj["first_public"]
            today = datetime.today().strftime("%Y-%m-%d")
            if self.hold_date:
                action_hold = ET.SubElement(actions, "ACTION")
                hold = ET.SubElement(action_hold, "HOLD")
                hold.set("HoldUntilDate", self.hold_date.strftime("%d-%m-%Y"))
            elif public > today and not self.hold_date:
                action_hold = ET.SubElement(actions, "ACTION")
                hold = ET.SubElement(action_hold, "HOLD")
                hold.set("HoldUntilDate", public)

            dom = minidom.parseString(ET.tostring(submission, encoding="utf-8"))
            submission_file.write(dom.toprettyxml().encode("utf-8"))

    def write(self):
        """
        Write registration and submission XML files.
        """
        self.write_study_xml()
        self.write_submission_xml()


@click.command(help="Study XML generation")
@click.version_option(__version__, message="assembly_uploader %(version)s")
@click.option("--study", required=True, help="Raw reads study ID")
@click.option(
    "--library",
    type=click.Choice(["metagenome", "metatranscriptome"], case_sensitive=False),
    required=True,
    help="Library type",
)
@click.option("--center", required=True, help="Center for upload e.g. EMG")
@click.option(
    "--hold",
    required=False,
    help="Hold date (private) in format dd-mm-yyyy. "
    "Will inherit the release date of the raw read study if not provided.",
)
@click.option(
    "--tpa",
    is_flag=True,
    default=False,
    help="Use this flag if the study is a third-party assembly. Default: False",
)
@click.option(
    "--publication",
    type=int,
    required=False,
    help="PubMed ID for connected publication if available",
)
@click.option(
    "--output-dir",
    type=click.Path(file_okay=False, dir_okay=True, writable=True),
    required=False,
    help="Path to output directory",
)
@click.option("--private", is_flag=True, default=False, help="Use flag if private")
@click.option(
    "--test",
    is_flag=True,
    default=False,
    help="Use flag when submitting to the ENA TEST server (adds a timestamp to the study alias)",
)
def main(study, library, center, hold, tpa, publication, output_dir, private, test):
    click.echo(f"Study: {study}")
    click.echo(f"Library: {library}")
    click.echo(f"Center: {center}")
    click.echo(f"Hold: {hold}")
    click.echo(f"TPA: {tpa}")
    click.echo(f"Publication: {publication}")
    click.echo(f"Output dir: {output_dir}")
    click.echo(f"Private: {private}")
    click.echo(f"Test: {test}")

    study_reg = StudyXMLGenerator(
        study=study,
        center_name=center,
        library=library,
        hold_date=hold,
        tpa=tpa,
        output_dir=Path(output_dir) if output_dir else None,
        publication=publication,
        private=private,
        test=test,
    )
    study_reg.write_study_xml()
    study_reg.write_submission_xml()


if __name__ == "__main__":
    main()



import importlib.metadata
import logging
import tempfile
import xml.etree.ElementTree as ET
from pathlib import Path
from xml.dom import minidom

import click
import requests

from assembly_uploader.submit_study import DROPBOX_DEV, DROPBOX_PROD
from assembly_uploader.webin_utils import (
    ensure_webin_credentials_exist,
    get_webin_credentials,
)

__version__ = importlib.metadata.version("assembly_uploader")


class StudyReleaseError(Exception):
    pass


def release_study(accession: str, is_test: bool = False, xml_path: Path = None):
    """
    Immediately publicly release a study, so that it is no longer private/embargoed/held in ENA.
    :param accession: study accession
    :param is_test: If true, will use wwwdev ENA Test Webin dropbox
    :param xml_path: Optional path to use for the submission xml file, otherwise a temp dir will be used.
    """
    with tempfile.TemporaryDirectory() as tmp_dir:
        xml_path = xml_path or Path(tmp_dir) / Path(accession + ".xml")

        with xml_path.open("wb") as release_file:
            submission = ET.Element("SUBMISSION")
            actions = ET.SubElement(submission, "ACTIONS")
            action_sub = ET.SubElement(actions, "ACTION")
            release = ET.SubElement(action_sub, "RELEASE")
            release.set("target", accession)
            dom = minidom.parseString(ET.tostring(submission, encoding="utf-8"))
            release_file.write(dom.toprettyxml().encode("utf-8"))

        endpoint = DROPBOX_DEV if is_test else DROPBOX_PROD

        files = {
            "SUBMISSION": xml_path.open("rb"),
        }

        release_report = requests.post(
            endpoint, files=files, auth=get_webin_credentials()
        )
        receipt_xml_str = release_report.content.decode("utf-8")
        if 'success="true"' in receipt_xml_str:
            logging.info(f"Released study {accession}")
        else:
            raise StudyReleaseError(f"Failed to release {accession}. {receipt_xml_str}")


@click.command(
    help="Release a private/held study on ENA, e.g. an uploaded assembly study"
)
@click.version_option(__version__, message="assembly_uploader %(version)s")
@click.option("--study", required=True, help="Study ID")
@click.option(
    "--xml_path", required=False, help="Path to use for the release XML submission"
)
@click.option(
    "--test",
    required=False,
    default=False,
    help="Use Webin Dev dropbox instead of prod",
    is_flag=True,
)
def main(study, test, xml_path):

    ensure_webin_credentials_exist()

    release_study(study, test, Path(xml_path))


if __name__ == "__main__":
    main()



import os

ENA_WEBIN = "ENA_WEBIN"
ENA_WEBIN_PASSWORD = "ENA_WEBIN_PASSWORD"


def ensure_webin_credentials_exist():
    if ENA_WEBIN not in os.environ:
        raise Exception(f"The variable {ENA_WEBIN} is missing from the env.")
    if ENA_WEBIN_PASSWORD not in os.environ:
        raise Exception(f"The variable {ENA_WEBIN_PASSWORD} is missing from the env")


def get_webin_credentials():
    ensure_webin_credentials_exist()
    webin = os.environ.get(ENA_WEBIN)
    password = os.environ.get(ENA_WEBIN_PASSWORD)
    return webin, password



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2024 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import logging
import os
import sys
from time import sleep

import requests
from requests.exceptions import ConnectionError, HTTPError, RequestException, Timeout

logging.basicConfig(level=logging.INFO)

RETRY_COUNT = 3


def get_default_connection_headers():
    return {
        "headers": {
            "Content-Type": "application/x-www-form-urlencoded",
            "Accept": "*/*",
        }
    }


def parse_accession(accession):
    if accession.startswith("PRJ"):
        return "study_accession"
    elif "RP" in accession:
        return "secondary_study_accession"
    elif "RR" in accession:
        return "run_accession"
    else:
        logging.error(f"{accession} is not a valid accession")
        sys.exit()


class EnaQuery:
    def __init__(self, accession, private=False):
        self.private_url = "https://www.ebi.ac.uk/ena/submit/report/"
        self.public_url = "https://www.ebi.ac.uk/ena/portal/api/search"
        self.accession = accession
        self.acc_type = parse_accession(accession)
        username = os.getenv("ENA_WEBIN")
        password = os.getenv("ENA_WEBIN_PASSWORD")
        if username is None or password is None:
            logging.error("ENA_WEBIN and ENA_WEBIN_PASSWORD are not set")
        if username and password:
            self.auth = (username, password)
        else:
            self.auth = None
        self.private = private

    def post_request(self, data):
        response = requests.post(
            self.public_url, data=data, **get_default_connection_headers()
        )
        return response

    def get_request(self, url):
        response = requests.get(url, auth=self.auth)
        return response

    def get_data_or_handle_error(self, response):
        try:
            data = json.loads(response.text)[0]
            if data is None:
                if self.private:
                    logging.error(
                        f"{self.accession} private data is not present in the specified Webin account"
                    )
                else:
                    logging.error(f"{self.accession} public data does not exist")
            else:
                return data
        except (IndexError, TypeError, ValueError, KeyError):
            logging.error(
                f"Failed to fetch {self.accession}, returned error: {response.text}"
            )

    def retry_or_handle_request_error(self, request, *args, **kwargs):
        attempt = 0
        while attempt < RETRY_COUNT:
            try:
                response = request(*args, **kwargs)
                response.raise_for_status()
                return response
            #   all other RequestExceptions are raised below
            except (Timeout, ConnectionError) as retry_err:
                attempt += 1
                if attempt >= RETRY_COUNT:
                    raise ValueError(
                        f"Could not find {self.accession} in ENA after {RETRY_COUNT} attempts. Error: {retry_err}"
                    )
                sleep(1)
            except HTTPError as http_err:
                print(f"HTTP response has an error status: {http_err}")
                raise
            except RequestException as req_err:
                print(f"Network-related error status: {req_err}")
                raise
            #   should hopefully encompass all other issues...
            except Exception as e:
                print(f"An unexpected error occurred: {e}")
                raise

    def _get_private_study(self):
        url = f"{self.private_url}studies/{self.accession}"
        response = self.retry_or_handle_request_error(self.get_request, url)
        study = self.get_data_or_handle_error(response)
        study_data = study["report"]
        try:
            reformatted_data = {
                "study_accession": study_data["secondaryId"],
                "study_title": study_data["title"],
                # remove time and keep date
                "first_public": study_data["firstPublic"].split("T")[0],
            }
        except AttributeError:
            # if the release date was postponed, firstPublic == None
            # fallback on holdDate
            reformatted_data = {
                "study_accession": study_data["secondaryId"],
                "study_title": study_data["title"],
                "first_public": study_data["holdDate"].split("T")[0],
            }
        logging.info(f"{self.accession} private study returned from ENA")
        return reformatted_data

    def _get_public_study(self):
        data = {
            "result": "study",
            "query": f'{self.acc_type}="{self.accession}"',
            "fields": "study_accession,study_title,first_public",
            "format": "json",
        }
        response = self.retry_or_handle_request_error(self.post_request, data)
        study = self.get_data_or_handle_error(response)
        logging.info(f"{self.accession} public study returned from ENA")
        return study

    def _get_private_run(self):
        url = f"{self.private_url}runs/{self.accession}"
        response = self.retry_or_handle_request_error(self.get_request, url)
        run = self.get_data_or_handle_error(response)
        run_data = run["report"]
        reformatted_data = {
            "run_accession": self.accession,
            "sample_accession": run_data["sampleId"],
            "instrument_model": run_data["instrumentModel"],
        }
        logging.info(f"{self.accession} private run returned from ENA")
        return reformatted_data

    def _get_public_run(self):
        data = {
            "result": "read_run",
            "query": f'run_accession="{self.accession}"',
            "fields": "run_accession,sample_accession,instrument_model",
            "format": "json",
        }
        response = self.retry_or_handle_request_error(self.post_request, data)
        run = self.get_data_or_handle_error(response)
        logging.info(f"{self.accession} public run returned from ENA")
        return run

    def build_query(self):
        if "study" in self.acc_type:
            if self.private:
                ena_response = self._get_private_study()
            else:
                ena_response = self._get_public_study()
        elif "run" in self.acc_type:
            if self.private:
                ena_response = self._get_private_run()
            else:
                ena_response = self._get_public_run()
        return ena_response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2024 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import importlib.metadata
import logging
import re
import xml.etree.ElementTree as ET
from pathlib import Path

import click
import requests

__version__ = importlib.metadata.version("assembly_uploader")


from assembly_uploader.webin_utils import (
    ensure_webin_credentials_exist,
    get_webin_credentials,
)

logging.basicConfig(level=logging.INFO)


DROPBOX_DEV = "https://wwwdev.ebi.ac.uk/ena/submit/drop-box/submit"
DROPBOX_PROD = "https://www.ebi.ac.uk/ena/submit/drop-box/submit/"


def parse_failed_study_acc(report):
    failed_re = (
        r"The object being added already exists in the submission account with accession: "
        r"\"(PRJ[EDN][A-Z][0-9]+)\""
    )
    root = ET.fromstring(report)
    errors = root.findall(".//ERROR")

    for e in errors:
        error_text = e.text
        existing_acc = re.findall(failed_re, error_text)
        if existing_acc:
            return existing_acc[0]


def parse_success_study_acc(report):
    success_re = r"accession=\"(PRJ[EDN][A-Z][0-9]+)\""
    new_acc = re.findall(success_re, report)
    if new_acc:
        return new_acc[0]


def submit_study(study_id: str, is_test: bool = False, directory: Path = None):
    endpoint = DROPBOX_DEV if is_test else DROPBOX_PROD
    logging.info(f"Submitting study xml {study_id}")
    workdir = directory or Path.cwd() / Path(f"{study_id}_upload")
    assert workdir.exists()

    submission_xml = workdir / Path(f"{study_id}_submission.xml")
    study_xml = workdir / Path(f"{study_id}_reg.xml")
    files = {
        "SUBMISSION": open(submission_xml, "rb"),
        "ACTION": (None, "ADD"),
        "PROJECT": open(study_xml, "rb"),
    }

    submission_report = requests.post(
        endpoint, files=files, auth=get_webin_credentials()
    )
    receipt_xml_str = submission_report.content.decode("utf-8")

    if 'success="true"' in receipt_xml_str:
        primary_accession = parse_success_study_acc(receipt_xml_str)
        logging.info(
            f"A new study accession has been created: {primary_accession}. Make a note of this!"
        )
        return primary_accession
    elif (
        "The object being added already exists in the submission account"
        in receipt_xml_str
    ):
        primary_accession = parse_failed_study_acc(receipt_xml_str)
        logging.info(
            f"An accession with this alias already exists in project {primary_accession}"
        )
        return primary_accession
    elif submission_report.status_code >= requests.codes.server_error:
        logging.error(
            "Project could not be registered on ENA as the server does not respond. Please again try later."
        )
    else:
        logging.error(
            f"Project could not be registered on ENA. HTTP response: {receipt_xml_str}"
        )


@click.command(help="Study submission")
@click.version_option(__version__, message="assembly_uploader %(version)s")
@click.option("--study", required=True, help="raw reads study ID")
@click.option("--directory", required=True, help="directory containing study XML")
@click.option(
    "--test",
    required=False,
    help="run test submission only",
    default=False,
    is_flag=True,
)
def main(study, test, directory):

    ensure_webin_credentials_exist()

    submit_study(study, test, Path(directory))


if __name__ == "__main__":
    main()
